import logging

# Configure the logging module
logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)

download_upload_logger = logging.getLogger("Download Upload Logger")



from enum import Enum
import os
import platform
import warnings


class DATA_ANALYSIS_OPTIONS_ENUM(Enum):
    PANDAS = "pandas"
    POLARS = "polars"


DATA_ANALYSIS_OPTIONS = DATA_ANALYSIS_OPTIONS_ENUM.PANDAS

# Determine the operating system
os_type = platform.system()

__DEFAULT_TMP_FOLDER = "pluto_sdk"

# Set the temporary folder based on the OS
match os_type:
    case "Windows":
        DEFAULT_TMP_PATH = os.path.join(os.environ["TEMP"], "pluto_sdk")
    case "Darwin":
        DEFAULT_TMP_PATH = f"/private/var/folders/{__DEFAULT_TMP_FOLDER}"
    case "Linux":
        DEFAULT_TMP_PATH = f"/tmp/{__DEFAULT_TMP_FOLDER}"
    case _:
        warnings.warn(
            f"Unsupported operating system: {os_type}. Files will be save locally in a folder called {__DEFAULT_TMP_FOLDER}"
        )
        DEFAULT_TMP_PATH = __DEFAULT_TMP_FOLDER



from .settings import DEFAULT_TMP_PATH
from typing import TYPE_CHECKING
from . import utils
import time
import os
from .log_handler import download_upload_logger
import requests
from . import api_endpoints

# TODO: Need to add results and methods to the API

if TYPE_CHECKING:
    from . import PlutoClient


class Attachments(list):
    def __init__(self, client: "PlutoClient") -> None:
        super().__init__()  # Initialize the list
        self._client = client


class Attachment:
    def __init__(self, client: "PlutoClient") -> None:
        self._client = client
        self.data_type = "attachment"
        self.uuid = ""
        self.filename = ""
        self.display_name = ""

    def list(self, experiment_id: str):
        data = {"data_type": self.data_type}
        response = self._client.get(
            f"{api_endpoints.EXPERIMENTS}/{experiment_id}/files",
            data=data,
        )
        attachments = Attachments(self._client)
        for attachment in response["attachments"]["items"]:
            attachment_as_object = utils.to_class(Attachment(self._client), attachment)
            attachments.append(attachment_as_object)

        return attachments

    def get(self, experiment_id: str, file_id: str):
        response = self.list(experiment_id)
        for attachment in response:
            if attachment.uuid == file_id:
                return attachment
            else:
                raise Exception("Attachment not found")

    def download(
        self,
        experiment_id: str,
        file_id: str,
        folder_path=DEFAULT_TMP_PATH,
        is_cache=True,
    ):
        attachment = self.get(experiment_id, file_id)
        name = (
            attachment.display_name if attachment.display_name else attachment.filename
        )
        data = {"filename": name}
        attachment_download = self._client.get(
            f"{api_endpoints.EXPERIMENTS}/{experiment_id}/files/{file_id}/download",
            params=data,
        )

        self._client._download_upload.download_file(
            experiment_id,
            os.path.join(folder_path, name),
            attachment_download["url"],
        )



from setuptools import setup, find_packages

setup(
    name="PlutoBio",
    version="0.1.11",
    author="Pluto Team",
    description="A Python SDK for Pluto.bio",
    packages=find_packages(exclude=["*tests*", "scripts", "examples"]),
    install_requires=["requests", "pandas"],
    python_requires=">=3.10, <4",
    keywords=["PlutoBio", "SDK", "API", "Biosciences", "Pluto"],
)
